#ifndef EL__TERMINAL_MAP_H
#define EL__TERMINAL_MAP_H

#ifdef __cplusplus
extern "C" {
#endif

int get_id_from_ID(unsigned int ID);
void set_id_ID(unsigned int id, unsigned int ID);
void remove_id(unsigned int id);

#ifdef __cplusplus
}
#endif

#endif /* EL__TERMINAL_MAP_H */
