#ifndef EL__SCRIPTING_LUA_LUA_H
#define EL__SCRIPTING_LUA_LUA_H

#ifdef __cplusplus
extern "C" {
#endif

struct module;

extern struct module lua_scripting_module;

#ifdef __cplusplus
}
#endif

#endif
